/** 
 * Program:  IsingAp.java
 * A simple program to calculate properties of the two-dimensional
 * IsingAp model using the Monte Carlo method.
 * The temperature is in units of the interaction strength J.
 **/

import java.awt.*;
import java.awt.event.*;
import java.lang.*;
import java.util.*;
import java.applet.*;

public class IsingAp extends Applet implements Runnable,ActionListener{

  // Define some class variables.
  long startTime,endTime;
  public static final int XDIM=30, YDIM=30;
  public static final int UP=1, DOWN=2, STRIPE=3, CHECKER=4;

  public static int FrameX=300,FrameY=300;  
  public static int IsingX=FrameX-100;
  public static int IsingY=FrameY-100;
  public static final double SPIN_UP=1.0;
  public static final double SPIN_DN=-1.0;
  public static final int DELAY=100;
  public static final double INIT_TEMP=2.;
  public static final double TEMPMIN=0.5;	
  public static final double TEMPMAX=100.0;

  public double temp=INIT_TEMP;
  
  double delta,spin_temp,energy_new,energy_old,ratio;

  String oldText;
  
  //These are some classes I've defined
  IsLattice lattice;
  IsCanvas canvas;
  
  //I'll put some buttons on the panel.
  Button b1,b2,b3;

  //Create a TextField object to hold the temperature value.
  TextField tempText;

  //Runs the program as a thread.
  Thread isThread=null;

  public void init(int istart) {
    IsingX=(IsingX/XDIM)*XDIM;
    IsingY=(IsingY/YDIM)*YDIM;
    lattice=new IsLattice(istart);

    //The frame is split into several panels.  BorderLayout() is a 
    //standard arrangement of panels.
    Panel bottomPanel=new Panel();
    Panel centerPanel=new Panel();
    setLayout(new BorderLayout());

    //Put the Ising display in the top panel.
    centerPanel.add(canvas=new IsCanvas(lattice,IsingX,IsingY));
    add("Center",centerPanel);
    //    add(canvas=new IsCanvas(lattice,FrameX,FrameY));

    //Put some useful buttons in the bottom panel.  Their functions are
    //defined below.
    b1=new Button("Stop");    
    b2=new Button("Pause");
    b3=new Button("Restart");
    b1.addActionListener(this);
    b2.addActionListener(this);
    b3.addActionListener(this);
    bottomPanel.add(b1);
    bottomPanel.add(b2);
    bottomPanel.add(b3);
	
    //Put the text field that holds the temperature in the bottom
    //panel.
    tempText=new TextField(""+temp,6);
    oldText=""+temp;
    tempText.addActionListener(this);	
    bottomPanel.add(tempText);	

    add("South",bottomPanel);
	
    if(isThread==null) {
      isThread=new Thread(this);
      isThread.start();
      isThread.setPriority(Thread.MIN_PRIORITY);
    }
  }
	
  // Buttons and text field are now activated and await action events.
  public void actionPerformed(ActionEvent event) {
    Object source=event.getSource();
    if(source==b1) {        // Hitting the "Stop" button disables the other 
      b2.setEnabled(false); // buttons.
      b3.setEnabled(false);
      tempText.setEnabled(false);
      stop();
    }
    if(source==b2) pause();    //pause() and restart() are defined below.
    if(source==b3) restart();
    if(source==tempText) setNewTemp();	
  }

  //The program is implemented as a thread, and the following command makes
  // it runnable.
  public void run(){
    Thread myThread=Thread.currentThread();
    while(isThread==myThread){
      startTime=System.currentTimeMillis();
      monteCarlo();
      canvas.repaint();
      try{
	endTime=System.currentTimeMillis();
	if((DELAY-(endTime-startTime))>0){ 
	  Thread.sleep(DELAY-(endTime-startTime));
	}else
	  Thread.sleep(DELAY);
      }
      catch(InterruptedException e) {}    
    }
  }

  //Function is executed when the pause button is clicked.
  public void pause() {
    isThread.suspend();
  }

  //Function is executed when the restart button is clicked.
  public void restart() {
    isThread.resume();
  }

  //Function is executed when the stop button is clicked.
  public void stop() {
    isThread=null;
  }

  //Function is executed when the value of the text field is modified.
  public void setNewTemp() {
    double val;
    try{
    	val=(double)Double.valueOf(tempText.getText()).doubleValue();
	if((val>=TEMPMIN)&&(val<=TEMPMAX)) {
		temp=val;
		oldText=""+val;
        }
	else {
	     	tempText.setText(oldText);
	}
    }
    catch (java.lang.NumberFormatException e){
	val=0.0;
	tempText.setText(oldText);

    }	
	
  }

  //This is the workhorse function for flipping the spins and updating
  //the configuration. 	
  public void monteCarlo() {
    
    for(int i=0;i<XDIM;i++) {
      int ip1=i+1;
      if(ip1>=XDIM) ip1-=XDIM;
      int im1=i-1;
      if(im1<0) im1+=XDIM;
      for(int j=0;j<YDIM;j++) {
	int jp1=j+1;
	if(jp1>=YDIM) jp1-=YDIM;
	int jm1=j-1;
	if(jm1<0) jm1+=YDIM;
	spin_temp=-lattice.spins[i][j];   //Flip spin
	energy_new=spin_temp*(lattice.spins[ip1][j]
			      +lattice.spins[i][jp1]
			      +lattice.spins[im1][j]
			      +lattice.spins[i][jm1]);
	energy_old=lattice.spins[i][j]*(lattice.spins[ip1][j] 
					+lattice.spins[i][jp1]
					+lattice.spins[im1][j]
					+lattice.spins[i][jm1]);
	delta=(energy_new-energy_old)/temp;
	if(delta>=0.0) {
	  lattice.spins[i][j]=spin_temp;
	}
	else {
	  ratio=Math.exp(delta);
	  if(ratio>=Math.random())
	    lattice.spins[i][j]=spin_temp; 
	}
      }
    }
  }
  
  //A main() to execute the program.
  public static void main(String[] args) {

    int istart=CHECKER;     //Starting configuration
    Frame f=new Frame();

    IsingAp itest=new IsingAp();    //itest is an IsingAp object
    
    //Set the frame size.  10 pix are used as borders, and the
    //header takes 30.

    
    //Make the frame closable
    f.addWindowListener(new WindowAdapter() {
      public void windowClosing(WindowEvent e) {System.exit(0);}
    });

    f.add(itest,"Center");
    itest.init(istart);

    f.setSize(FrameX+10,FrameY+30);        
    f.setVisible(true);    //Display everything
    itest.run();
  }
}

/**
 * Program IsCanvas.java
 * This program plots the spins.  A grid is drawn and spins are
 * labels red for up, green for down.  Very Christmasy.
 * This program can be separated from the main file.  Call it IsCanvas.java
 * and make the class file with javac IsCanvas.java.
 * Also, add "import java.awt.*;" as the first line (drop the quotes).
 **/

class IsCanvas extends Canvas {
  
  int IsingX,IsingY;
  int ScaleX,ScaleY;
  double dw,dh;
  int w,h;
  IsLattice lattice;

  //Needed for double buffering the image.
  Dimension offDimension;
  Image offImage;
  Graphics offGraphics;

  //A constructor
  public IsCanvas(IsLattice lattice,int IsingX,int IsingY) {
    this.lattice=lattice;
    this.IsingX=IsingX;
    this.IsingY=IsingY;
  }

  //Another constructor
  public IsCanvas() {;}
    
  // Define the local paint method to draw the grid and color in the squares
  public void paint(Graphics g) {
    update(g);
  }

  public void update(Graphics g) {

    //Size of the window.  The size returned by the getSize() functions
    //is slightly smaller than the actual size used by the setSize() method
    //of Frame.  Thus, we have to scale the spacing to make a uniform
    //grid.

    w = getSize().width;
    h = getSize().height;
    ScaleX=IsingX/w;    
    ScaleY=IsingY/h;

    Dimension d=getSize();
    
    //Spacing between lines
    dw=(double)w/(double)(IsingAp.XDIM);
    dh=(double)h/(double)(IsingAp.YDIM);
    
    //Create the offscreen image
    if((offGraphics==null)
       ||(d.width!=offDimension.width)
       ||(d.height!=offDimension.height)){
      offDimension=d;
      offImage=createImage(d.width,d.height);
      offGraphics=offImage.getGraphics();
    }

    //Erase the previous image
    offGraphics.setColor(getBackground());
    offGraphics.fillRect(0,0,d.width,d.height);
    offGraphics.setColor(Color.black);
    
    //Fill in the squares.  Image is drawn in the background.
    for(int i=0;i<IsingAp.XDIM;i++)
      for(int j=0;j<IsingAp.YDIM;j++) {
	if(lattice.spins[i][j]==IsingAp.SPIN_UP) {
	  offGraphics.setColor(Color.red);
	  offGraphics.fillRect(i*(int)dw,j*(int)dh,(i+1)*(int)dw,
			       (j+1)*(int)dh);
	}
	else {
	  offGraphics.clearRect(i*(int)dw,j*(int)dh,(i+1)*(int)dw,
				(j+1)*(int)dh);
	}
      }

    //Now draw the completed image on the canvas.
    g.drawImage(offImage,0,0,this);
  }

  //If we don't specify this, the canvas might not show up at all
  //(depending on the layout manager).
  public Dimension getMinimumSize() {
    return new Dimension(IsingX,IsingY);
  }
  
  //If we don't specify this, the canvas might not show up at all
  //(depending on the layout manager).
  public Dimension getPreferredSize() {
    return getMinimumSize();
  }
}

/**
 * Program IsLattice.java
 * This program sets up the lattice.
 **/

class IsLattice extends IsingAp{
  
  double spins[][]=new double[IsingAp.XDIM][IsingAp.YDIM];  
  //  public static final int UP=1, DOWN=2, STRIPE=3;
  
  public IsLattice(int istart){
    double spin_old=IsingAp.SPIN_DN;
    switch(istart){
    case UP:
      for(int i=0;i<IsingAp.XDIM;i++)
	for(int j=0;j<IsingAp.YDIM;j++)	
	  spins[i][j]=IsingAp.SPIN_UP;
      break;
      
    case DOWN:
      for(int i=0;i<IsingAp.XDIM;i++)
	for(int j=0;j<IsingAp.YDIM;j++)	
	  spins[i][j]=IsingAp.SPIN_DN;
      break;
      
    case STRIPE:
      for(int i=0;i<IsingAp.XDIM;i++)
	for(int j=0;j<IsingAp.YDIM;j++){
	  if(spin_old==IsingAp.SPIN_DN){
	    spins[i][j]=IsingAp.SPIN_UP;
	    spin_old=IsingAp.SPIN_UP;
	  }
	  else {
	    spins[i][j]=IsingAp.SPIN_DN;
	    spin_old=IsingAp.SPIN_DN;
	  }
	}
      break;
      
    case CHECKER:
      for(int i=0;i<IsingAp.XDIM;i++) {
	for(int j=0;j<IsingAp.YDIM;j++){
	  if(spin_old==IsingAp.SPIN_DN){
	    spins[i][j]=IsingAp.SPIN_UP;
	    spin_old=IsingAp.SPIN_UP;
	  }
	  else {
	    spins[i][j]=IsingAp.SPIN_DN;
	    spin_old=IsingAp.SPIN_DN;
	  }
	}
	//Start next column with opposite spin.
	if(spin_old==IsingAp.SPIN_DN)
	  spin_old=IsingAp.SPIN_UP;
	else
	  spin_old=IsingAp.SPIN_DN;
      }
      break;
      
    default:
      System.out.println("Illegal option");
      break;
    }
  }

  
}
    





